#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import os
import sys

import pytest


def main():
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'tests.settings')
    return pytest.main()


if __name__ == '__main__':
    sys.exit(main())



from __future__ import absolute_import

from contextlib import contextmanager

from django.test.utils import modify_settings

from corsheaders.signals import check_request_enabled


def add_middleware(action, path):
    return modify_settings(**{
        'MIDDLEWARE': {
            action: path,
        }
    })


def append_middleware(path):
    return add_middleware('append', path)


def prepend_middleware(path):
    return add_middleware('prepend', path)


@contextmanager
def temporary_check_request_hander(handler):
    check_request_enabled.connect(handler)
    try:
        yield
    finally:
        check_request_enabled.disconnect(handler)



from __future__ import absolute_import

from django.http import HttpResponse
from django.test import TestCase
from django.test.utils import override_settings
from django.utils.deprecation import MiddlewareMixin

from corsheaders.middleware import (
    ACCESS_CONTROL_ALLOW_CREDENTIALS, ACCESS_CONTROL_ALLOW_HEADERS, ACCESS_CONTROL_ALLOW_METHODS,
    ACCESS_CONTROL_ALLOW_ORIGIN, ACCESS_CONTROL_EXPOSE_HEADERS, ACCESS_CONTROL_MAX_AGE
)
from tests.utils import append_middleware, prepend_middleware, temporary_check_request_hander


class ShortCircuitMiddleware(MiddlewareMixin):

    def process_request(self, request):
        return HttpResponse('short-circuit-middleware-response')


class CorsMiddlewareTests(TestCase):

    def test_get_no_origin(self):
        resp = self.client.get('/')
        assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    def test_get_origin_vary_by_default(self):
        resp = self.client.get('/')
        assert resp['Vary'] == 'Origin'

    @override_settings(CORS_ORIGIN_WHITELIST=['http://example.com'])
    def test_get_not_in_whitelist(self):
        resp = self.client.get('/', HTTP_ORIGIN='http://example.org')
        assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    @override_settings(CORS_ORIGIN_WHITELIST=['https://example.org'])
    def test_get_not_in_whitelist_due_to_wrong_scheme(self):
        resp = self.client.get('/', HTTP_ORIGIN='http://example.org')
        assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    @override_settings(CORS_ORIGIN_WHITELIST=['http://example.com', 'http://example.org'])
    def test_get_in_whitelist(self):
        resp = self.client.get('/', HTTP_ORIGIN='http://example.org')
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == 'http://example.org'

    @override_settings(CORS_ORIGIN_WHITELIST=['http://example.com', 'null'])
    def test_null_in_whitelist(self):
        resp = self.client.get('/', HTTP_ORIGIN='null')
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == 'null'

    @override_settings(
        CORS_ORIGIN_ALLOW_ALL=True,
        CORS_EXPOSE_HEADERS=['accept', 'origin', 'content-type'],
    )
    def test_get_expose_headers(self):
        resp = self.client.get('/', HTTP_ORIGIN='http://example.com')
        assert resp[ACCESS_CONTROL_EXPOSE_HEADERS] == 'accept, origin, content-type'

    @override_settings(CORS_ORIGIN_ALLOW_ALL=True)
    def test_get_dont_expose_headers(self):
        resp = self.client.get('/', HTTP_ORIGIN='http://example.com')
        assert ACCESS_CONTROL_EXPOSE_HEADERS not in resp

    @override_settings(
        CORS_ALLOW_CREDENTIALS=True,
        CORS_ORIGIN_ALLOW_ALL=True,
    )
    def test_get_allow_credentials(self):
        resp = self.client.get('/', HTTP_ORIGIN='http://example.com')
        assert resp[ACCESS_CONTROL_ALLOW_CREDENTIALS] == 'true'

    @override_settings(CORS_ORIGIN_ALLOW_ALL=True)
    def test_get_dont_allow_credentials(self):
        resp = self.client.get('/', HTTP_ORIGIN='http://example.com')
        assert ACCESS_CONTROL_ALLOW_CREDENTIALS not in resp

    @override_settings(
        CORS_ALLOW_HEADERS=['content-type', 'origin'],
        CORS_ALLOW_METHODS=['GET', 'OPTIONS'],
        CORS_PREFLIGHT_MAX_AGE=1002,
        CORS_ORIGIN_ALLOW_ALL=True,
    )
    def test_options_allowed_origin(self):
        resp = self.client.options(
            '/',
            HTTP_ORIGIN='http://example.com',
        )
        assert resp[ACCESS_CONTROL_ALLOW_HEADERS] == 'content-type, origin'
        assert resp[ACCESS_CONTROL_ALLOW_METHODS] == 'GET, OPTIONS'
        assert resp[ACCESS_CONTROL_MAX_AGE] == '1002'

    @override_settings(
        CORS_ALLOW_HEADERS=['content-type', 'origin'],
        CORS_ALLOW_METHODS=['GET', 'OPTIONS'],
        CORS_PREFLIGHT_MAX_AGE=0,
        CORS_ORIGIN_ALLOW_ALL=True,
    )
    def test_options_no_max_age(self):
        resp = self.client.options('/', HTTP_ORIGIN='http://example.com')
        assert resp[ACCESS_CONTROL_ALLOW_HEADERS] == 'content-type, origin'
        assert resp[ACCESS_CONTROL_ALLOW_METHODS] == 'GET, OPTIONS'
        assert ACCESS_CONTROL_MAX_AGE not in resp

    @override_settings(
        CORS_ALLOW_METHODS=['OPTIONS'],
        CORS_ALLOW_CREDENTIALS=True,
        CORS_ORIGIN_WHITELIST=['http://localhost:9000'],
    )
    def test_options_whitelist_with_port(self):
        resp = self.client.options('/', HTTP_ORIGIN='http://localhost:9000')
        assert resp[ACCESS_CONTROL_ALLOW_CREDENTIALS] == 'true'

    @override_settings(
        CORS_ALLOW_METHODS=['OPTIONS'],
        CORS_ALLOW_CREDENTIALS=True,
        CORS_ORIGIN_REGEX_WHITELIST=[r'^http?://(\w+\.)?example\.com$'],
    )
    def test_options_adds_origin_when_domain_found_in_origin_regex_whitelist(self):
        resp = self.client.options('/', HTTP_ORIGIN='http://foo.example.com')
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == 'http://foo.example.com'

    @override_settings(
        CORS_ALLOW_METHODS=['OPTIONS'],
        CORS_ALLOW_CREDENTIALS=True,
        CORS_ORIGIN_REGEX_WHITELIST=(r'^http?://(\w+\.)?example\.org$',),
    )
    def test_options_will_not_add_origin_when_domain_not_found_in_origin_regex_whitelist(self):
        resp = self.client.options('/', HTTP_ORIGIN='http://foo.example.com')
        assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    def test_options(self):
        resp = self.client.options(
            '/',
            HTTP_ACCESS_CONTROL_REQUEST_METHOD='value',
        )
        assert resp.status_code == 200

    def test_options_empty_request_method(self):
        resp = self.client.options(
            '/',
            HTTP_ACCESS_CONTROL_REQUEST_METHOD='',
        )
        assert resp.status_code == 200

    def test_options_no_header(self):
        resp = self.client.options('/')
        assert resp.status_code == 404

    @override_settings(
        CORS_ALLOW_CREDENTIALS=True,
        CORS_ORIGIN_ALLOW_ALL=True,
    )
    def test_allow_all_origins_get(self):
        resp = self.client.get('/', HTTP_ORIGIN='http://example.com')
        assert resp.status_code == 200
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == 'http://example.com'
        assert resp['Vary'] == 'Origin'

    @override_settings(
        CORS_ALLOW_CREDENTIALS=True,
        CORS_ORIGIN_ALLOW_ALL=True,
    )
    def test_allow_all_origins_options(self):
        resp = self.client.options(
            '/',
            HTTP_ORIGIN='http://example.com',
            HTTP_ACCESS_CONTROL_REQUEST_METHOD='value',
        )
        assert resp.status_code == 200
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == 'http://example.com'
        assert resp['Vary'] == 'Origin'

    @override_settings(
        CORS_ALLOW_CREDENTIALS=True,
        CORS_ORIGIN_ALLOW_ALL=True,
    )
    def test_non_200_headers_still_set(self):
        """
        It's not clear whether the header should still be set for non-HTTP200
        when not a preflight request. However this is the existing behaviour for
        django-cors-middleware, so at least this test makes that explicit, especially
        since for the switch to Django 1.10, special-handling will need to be put in
        place to preserve this behaviour. See `ExceptionMiddleware` mention here:
        https://docs.djangoproject.com/en/1.10/topics/http/middleware/#upgrading-pre-django-1-10-style-middleware
        """
        resp = self.client.get('/test-401/', HTTP_ORIGIN='http://example.com')
        assert resp.status_code == 401
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == 'http://example.com'

    @override_settings(
        CORS_ALLOW_CREDENTIALS=True,
        CORS_ORIGIN_ALLOW_ALL=True,
    )
    def test_auth_view_options(self):
        """
        Ensure HTTP200 and header still set, for preflight requests to views requiring
        authentication. See: https://github.com/ottoyiu/django-cors-headers/issues/3
        """
        resp = self.client.options(
            '/test-401/',
            HTTP_ORIGIN='http://example.com',
            HTTP_ACCESS_CONTROL_REQUEST_METHOD='value',
        )
        assert resp.status_code == 200
        assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == 'http://example.com'
        assert resp['Content-Length'] == '0'

    def test_signal_handler_that_returns_false(self):
        def handler(*args, **kwargs):
            return False

        with temporary_check_request_hander(handler):
            resp = self.client.options(
                '/',
                HTTP_ORIGIN='http://example.com',
                HTTP_ACCESS_CONTROL_REQUEST_METHOD='value',
            )

            assert resp.status_code == 200
            assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    def test_signal_handler_that_returns_true(self):
        def handler(*args, **kwargs):
            return True

        with temporary_check_request_hander(handler):
            resp = self.client.options(
                '/',
                HTTP_ORIGIN='http://example.com',
                HTTP_ACCESS_CONTROL_REQUEST_METHOD='value',
            )
            assert resp.status_code == 200
            assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == 'http://example.com'

    @override_settings(CORS_ORIGIN_WHITELIST=['http://example.com'])
    def test_signal_handler_allow_some_urls_to_everyone(self):
        def allow_api_to_all(sender, request, **kwargs):
            return request.path.startswith('/api/')

        with temporary_check_request_hander(allow_api_to_all):
            resp = self.client.options(
                '/',
                HTTP_ORIGIN='http://example.org',
                HTTP_ACCESS_CONTROL_REQUEST_METHOD='value',
            )
            assert resp.status_code == 200
            assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

            resp = self.client.options(
                '/api/something/',
                HTTP_ORIGIN='http://example.org',
                HTTP_ACCESS_CONTROL_REQUEST_METHOD='value',
            )
            assert resp.status_code == 200
            assert resp[ACCESS_CONTROL_ALLOW_ORIGIN] == 'http://example.org'

    @override_settings(CORS_ORIGIN_WHITELIST=['http://example.com'])
    def test_signal_called_once_during_normal_flow(self):
        def allow_all(sender, request, **kwargs):
            allow_all.calls += 1
            return True
        allow_all.calls = 0

        with temporary_check_request_hander(allow_all):
            self.client.get('/', HTTP_ORIGIN='http://example.org')

            assert allow_all.calls == 1

    @override_settings(CORS_ORIGIN_WHITELIST=['http://example.com'])
    @prepend_middleware('tests.test_middleware.ShortCircuitMiddleware')
    def test_get_short_circuit(self):
        """
        Test a scenario when a middleware that returns a response is run before
        the ``CorsMiddleware``. In this case
        ``CorsMiddleware.process_response()`` should ignore the request if
        MIDDLEWARE setting is used (new mechanism in Django 1.10+).
        """
        resp = self.client.get('/', HTTP_ORIGIN='http://example.com')
        assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    @override_settings(
        CORS_ORIGIN_WHITELIST=['http://example.com'],
        CORS_URLS_REGEX=r'^/foo/$',
    )
    @prepend_middleware(__name__ + '.ShortCircuitMiddleware')
    def test_get_short_circuit_should_be_ignored(self):
        resp = self.client.get('/', HTTP_ORIGIN='http://example.com')
        assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    @override_settings(
        CORS_ORIGIN_WHITELIST=['http://example.com'],
        CORS_URLS_REGEX=r'^/foo/$',
    )
    def test_get_regex_matches(self):
        resp = self.client.get('/foo/', HTTP_ORIGIN='http://example.com')
        assert ACCESS_CONTROL_ALLOW_ORIGIN in resp

    @override_settings(
        CORS_ORIGIN_WHITELIST=['http://example.com'],
        CORS_URLS_REGEX=r'^/not-foo/$',
    )
    def test_get_regex_doesnt_match(self):
        resp = self.client.get('/foo/', HTTP_ORIGIN='http://example.com')
        assert ACCESS_CONTROL_ALLOW_ORIGIN not in resp

    @override_settings(
        CORS_ORIGIN_WHITELIST=['http://example.com'],
        CORS_URLS_REGEX=r'^/foo/$',
    )
    def test_get_regex_matches_path_info(self):
        resp = self.client.get('/foo/', HTTP_ORIGIN='http://example.com', SCRIPT_NAME='/prefix/')
        assert ACCESS_CONTROL_ALLOW_ORIGIN in resp

    @override_settings(CORS_ORIGIN_WHITELIST=['http://example.com'])
    def test_cors_enabled_is_attached_and_bool(self):
        """
        Ensure that request._cors_enabled is available - although a private API
        someone might use it for debugging
        """
        resp = self.client.get('/', HTTP_ORIGIN='http://example.com')
        request = resp.wsgi_request
        assert isinstance(request._cors_enabled, bool)
        assert request._cors_enabled

    @override_settings(CORS_ORIGIN_WHITELIST=['http://example.com'])
    def test_works_if_view_deletes_cors_enabled(self):
        """
        Just in case something crazy happens in the view or other middleware,
        check that get_response doesn't fall over if `_cors_enabled` is removed
        """
        resp = self.client.get(
            '/delete-is-enabled/',
            HTTP_ORIGIN='http://example.com',
        )
        assert ACCESS_CONTROL_ALLOW_ORIGIN in resp


@override_settings(
    CORS_REPLACE_HTTPS_REFERER=True,
    CORS_ORIGIN_REGEX_WHITELIST=(r'.*example.*',),
)
class RefererReplacementCorsMiddlewareTests(TestCase):

    def test_get_replaces_referer_when_secure(self):
        resp = self.client.get(
            '/',
            HTTP_FAKE_SECURE='true',
            HTTP_HOST='example.com',
            HTTP_ORIGIN='https://example.org',
            HTTP_REFERER='https://example.org/foo'
        )
        assert resp.status_code == 200
        assert resp.wsgi_request.META['HTTP_REFERER'] == 'https://example.com/'
        assert resp.wsgi_request.META['ORIGINAL_HTTP_REFERER'] == 'https://example.org/foo'

    @append_middleware('corsheaders.middleware.CorsPostCsrfMiddleware')
    def test_get_post_middleware_rereplaces_referer_when_secure(self):
        resp = self.client.get(
            '/',
            HTTP_FAKE_SECURE='true',
            HTTP_HOST='example.com',
            HTTP_ORIGIN='https://example.org',
            HTTP_REFERER='https://example.org/foo'
        )
        assert resp.status_code == 200
        assert resp.wsgi_request.META['HTTP_REFERER'] == 'https://example.org/foo'
        assert 'ORIGINAL_HTTP_REFERER' not in resp.wsgi_request.META

    def test_get_does_not_replace_referer_when_insecure(self):
        resp = self.client.get(
            '/',
            HTTP_HOST='example.com',
            HTTP_ORIGIN='https://example.org',
            HTTP_REFERER='https://example.org/foo'
        )
        assert resp.status_code == 200
        assert resp.wsgi_request.META['HTTP_REFERER'] == 'https://example.org/foo'
        assert 'ORIGINAL_HTTP_REFERER' not in resp.wsgi_request.META

    @override_settings(CORS_REPLACE_HTTPS_REFERER=False)
    def test_get_does_not_replace_referer_when_disabled(self):
        resp = self.client.get(
            '/',
            HTTP_FAKE_SECURE='true',
            HTTP_HOST='example.com',
            HTTP_ORIGIN='https://example.org',
            HTTP_REFERER='https://example.org/foo',
        )
        assert resp.status_code == 200
        assert resp.wsgi_request.META['HTTP_REFERER'] == 'https://example.org/foo'
        assert 'ORIGINAL_HTTP_REFERER' not in resp.wsgi_request.META

    def test_get_does_not_fail_in_referer_replacement_when_referer_missing(self):
        resp = self.client.get(
            '/',
            HTTP_FAKE_SECURE='true',
            HTTP_HOST='example.com',
            HTTP_ORIGIN='https://example.org',
        )
        assert resp.status_code == 200
        assert 'HTTP_REFERER' not in resp.wsgi_request.META
        assert 'ORIGINAL_HTTP_REFERER' not in resp.wsgi_request.META

    def test_get_does_not_fail_in_referer_replacement_when_host_missing(self):
        resp = self.client.get(
            '/',
            HTTP_FAKE_SECURE='true',
            HTTP_ORIGIN='https://example.org',
            HTTP_REFERER='https://example.org/foo',
        )
        assert resp.status_code == 200
        assert resp.wsgi_request.META['HTTP_REFERER'] == 'https://example.org/foo'
        assert 'ORIGINAL_HTTP_REFERER' not in resp.wsgi_request.META

    @override_settings(CORS_ORIGIN_REGEX_WHITELIST=())
    def test_get_does_not_replace_referer_when_not_valid_request(self):
        resp = self.client.get(
            '/',
            HTTP_FAKE_SECURE='true',
            HTTP_HOST='example.com',
            HTTP_ORIGIN='https://example.org',
            HTTP_REFERER='https://example.org/foo',
        )
        assert resp.status_code == 200
        assert resp.wsgi_request.META['HTTP_REFERER'] == 'https://example.org/foo'
        assert 'ORIGINAL_HTTP_REFERER' not in resp.wsgi_request.META



from __future__ import absolute_import

import re

import pytest
from django.core.checks import Error
from django.core.management import base, call_command
from django.test import SimpleTestCase
from django.test.utils import override_settings

from corsheaders.checks import check_settings


class ChecksTests(SimpleTestCase):

    def check_error_codes(self, expected):
        errors = check_settings([])
        assert len(errors) == len(expected)
        assert all(isinstance(e, Error) for e in errors)
        assert [e.id for e in errors] == expected

    def test_defaults_pass(self):
        self.check_error_codes([])

    def test_defaults_pass_check(self):
        call_command('check')

    @override_settings(CORS_ORIGIN_ALLOW_ALL=object)
    def test_checks_are_bound(self):
        with pytest.raises(base.SystemCheckError):
            call_command('check')

    @override_settings(CORS_ALLOW_HEADERS=object)
    def test_cors_allow_headers_non_sequence(self):
        self.check_error_codes(['corsheaders.E001'])

    @override_settings(CORS_ALLOW_HEADERS=[object])
    def test_cors_allow_headers_non_string(self):
        self.check_error_codes(['corsheaders.E001'])

    @override_settings(CORS_ALLOW_METHODS=object)
    def test_cors_allow_methods_non_sequence(self):
        self.check_error_codes(['corsheaders.E002'])

    @override_settings(CORS_ALLOW_METHODS=[object])
    def test_cors_allow_methods_non_string(self):
        self.check_error_codes(['corsheaders.E002'])

    @override_settings(CORS_ALLOW_CREDENTIALS=object)
    def test_cors_allow_credentials_non_bool(self):
        self.check_error_codes(['corsheaders.E003'])

    @override_settings(CORS_PREFLIGHT_MAX_AGE='10')
    def test_cors_preflight_max_age_non_integer(self):
        self.check_error_codes(['corsheaders.E004'])

    @override_settings(CORS_PREFLIGHT_MAX_AGE=-1)
    def test_cors_preflight_max_age_negative(self):
        self.check_error_codes(['corsheaders.E004'])

    @override_settings(CORS_ORIGIN_ALLOW_ALL=object)
    def test_cors_origin_allow_all_non_bool(self):
        self.check_error_codes(['corsheaders.E005'])

    @override_settings(CORS_ORIGIN_WHITELIST=object)
    def test_cors_origin_whitelist_non_sequence(self):
        self.check_error_codes(['corsheaders.E006'])

    @override_settings(CORS_ORIGIN_WHITELIST=[object])
    def test_cors_origin_whitelist_non_string(self):
        self.check_error_codes(['corsheaders.E006'])

    @override_settings(CORS_ORIGIN_WHITELIST=['http://example.com', 'null'])
    def test_cors_origin_whitelist_allowed(self):
        self.check_error_codes([])

    @override_settings(CORS_ORIGIN_WHITELIST=['example.com'])
    def test_cors_origin_whitelist_no_scheme(self):
        self.check_error_codes(['corsheaders.E013'])

    @override_settings(CORS_ORIGIN_WHITELIST=['https://'])
    def test_cors_origin_whitelist_no_netloc(self):
        self.check_error_codes(['corsheaders.E013'])

    @override_settings(CORS_ORIGIN_WHITELIST=['https://example.com/foobar'])
    def test_cors_origin_whitelist_path(self):
        self.check_error_codes(['corsheaders.E014'])

    @override_settings(CORS_ORIGIN_REGEX_WHITELIST=object)
    def test_cors_origin_regex_whitelist_non_sequence(self):
        self.check_error_codes(['corsheaders.E007'])

    @override_settings(CORS_ORIGIN_REGEX_WHITELIST=[re.compile(r'a')])
    def test_cors_origin_regex_whitelist_regex(self):
        self.check_error_codes([])

    @override_settings(CORS_EXPOSE_HEADERS=object)
    def test_cors_expose_headers_non_sequence(self):
        self.check_error_codes(['corsheaders.E008'])

    @override_settings(CORS_EXPOSE_HEADERS=[object])
    def test_cors_expose_headers_non_string(self):
        self.check_error_codes(['corsheaders.E008'])

    @override_settings(CORS_URLS_REGEX=object)
    def test_cors_urls_regex_non_string(self):
        self.check_error_codes(['corsheaders.E009'])

    @override_settings(CORS_REPLACE_HTTPS_REFERER=object)
    def test_cors_replace_https_referer_failure(self):
        self.check_error_codes(['corsheaders.E011'])

    @override_settings(CORS_MODEL='something')
    def test_cors_model_failure(self):
        self.check_error_codes(['corsheaders.E012'])



from __future__ import absolute_import

SECRET_KEY = 'NOTASECRET'

ALLOWED_HOSTS = ['*']

INSTALLED_APPS = [
    'corsheaders',
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
    },
}

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
    },
]

ROOT_URLCONF = 'tests.urls'

MIDDLEWARE = [
    'corsheaders.middleware.CorsMiddleware',
]

SECURE_PROXY_SSL_HEADER = ('HTTP_FAKE_SECURE', 'true')



from __future__ import absolute_import

from django.conf.urls import url
from django.http import Http404, HttpResponse


def test_view(request):
    if request.method != 'GET':
        raise Http404()
    return HttpResponse("Test view")


def test_view_http401(request):
    return HttpResponse('Unauthorized', status=401)


def test_view_that_deletes_is_enabled(request):
    if hasattr(request, '_cors_enabled'):
        del request._cors_enabled
    return HttpResponse()


urlpatterns = [
    url(r'^$', test_view),
    url(r'^foo/$', test_view),
    url(r'^test-401/$', test_view_http401),
    url(r'^delete-is-enabled/$', test_view_that_deletes_is_enabled),
]



from __future__ import absolute_import

import re
from numbers import Integral

from django.conf import settings
from django.core import checks
from django.utils import six
from django.utils.six.moves.urllib.parse import urlparse

from corsheaders.conf import conf

try:
    from collections.abc import Sequence  # Python 3.3+
except ImportError:  # pragma: no cover
    from collections import Sequence

re_type = type(re.compile(''))


@checks.register
def check_settings(app_configs, **kwargs):
    errors = []

    if not is_sequence(conf.CORS_ALLOW_HEADERS, six.string_types):
        errors.append(
            checks.Error(
                "CORS_ALLOW_HEADERS should be a sequence of strings.",
                id="corsheaders.E001"
            )
        )

    if not is_sequence(conf.CORS_ALLOW_METHODS, six.string_types):
        errors.append(
            checks.Error(
                "CORS_ALLOW_METHODS should be a sequence of strings.",
                id="corsheaders.E002"
            )
        )

    if not isinstance(conf.CORS_ALLOW_CREDENTIALS, bool):
        errors.append(
            checks.Error(
                "CORS_ALLOW_CREDENTIALS should be a bool.",
                id="corsheaders.E003"
            )
        )

    if not isinstance(conf.CORS_PREFLIGHT_MAX_AGE, Integral) or conf.CORS_PREFLIGHT_MAX_AGE < 0:
        errors.append(
            checks.Error(
                "CORS_PREFLIGHT_MAX_AGE should be an integer greater than or equal to zero.",
                id="corsheaders.E004"
            )
        )

    if not isinstance(conf.CORS_ORIGIN_ALLOW_ALL, bool):
        errors.append(
            checks.Error(
                "CORS_ORIGIN_ALLOW_ALL should be a bool.",
                id="corsheaders.E005"
            )
        )

    if not is_sequence(conf.CORS_ORIGIN_WHITELIST, six.string_types):
        errors.append(
            checks.Error(
                "CORS_ORIGIN_WHITELIST should be a sequence of strings.",
                id="corsheaders.E006"
            )
        )
    else:
        for origin in conf.CORS_ORIGIN_WHITELIST:
            if origin == 'null':
                continue
            parsed = urlparse(origin)
            if parsed.scheme == '' or parsed.netloc == '':
                errors.append(checks.Error(
                    "Origin {} in CORS_ORIGIN_WHITELIST is missing scheme or netloc".format(repr(origin)),
                    id="corsheaders.E013"
                ))
            else:
                # Only do this check in this case because if the scheme is not provided, netloc ends up in path
                for part in ('path', 'params', 'query', 'fragment'):
                    if getattr(parsed, part) != '':
                        errors.append(checks.Error(
                            "Origin {} in CORS_ORIGIN_WHITELIST should not have {}".format(repr(origin), part),
                            id="corsheaders.E014"
                        ))

    if not is_sequence(conf.CORS_ORIGIN_REGEX_WHITELIST, six.string_types + (re_type,)):
        errors.append(
            checks.Error(
                "CORS_ORIGIN_REGEX_WHITELIST should be a sequence of strings and/or compiled regexes.",
                id="corsheaders.E007"
            )
        )

    if not is_sequence(conf.CORS_EXPOSE_HEADERS, six.string_types):
        errors.append(
            checks.Error(
                "CORS_EXPOSE_HEADERS should be a sequence.",
                id="corsheaders.E008"
            )
        )

    if not isinstance(conf.CORS_URLS_REGEX, six.string_types + (re_type,)):
        errors.append(
            checks.Error(
                "CORS_URLS_REGEX should be a string or regex.",
                id="corsheaders.E009"
            )
        )

    if not isinstance(conf.CORS_REPLACE_HTTPS_REFERER, bool):
        errors.append(
            checks.Error(
                "CORS_REPLACE_HTTPS_REFERER should be a bool.",
                id="corsheaders.E011"
            )
        )

    if hasattr(settings, 'CORS_MODEL'):
        errors.append(
            checks.Error(
                "The CORS_MODEL setting has been removed - see django-cors-headers' HISTORY.",
                id="corsheaders.E012"
            )
        )

    return errors


def is_sequence(thing, types):
    return (
        isinstance(thing, Sequence)
        and all(isinstance(x, types) for x in thing)
    )



from __future__ import absolute_import

from django.test import SimpleTestCase
from django.test.utils import override_settings

from corsheaders.conf import conf


class ConfTests(SimpleTestCase):

    @override_settings(CORS_ALLOW_HEADERS=['foo'])
    def test_can_override(self):
        assert conf.CORS_ALLOW_HEADERS == ['foo']
